#include <stdio.h>
#include <stdlib.h>

static int deterministic_flag = 1;


int main(int argc, char **argv)
{

  FILE * fp = stdout;
  int num_of_rooms = 4;
  int i,j;
  int seed;
  char fname[100];
  char * err[1];
  

  if ((argc <= 1) || (argc >3)) {
    fprintf(stderr,"Usage: %s #of_rooms\n",argv[0]);
    exit(1);
  }
  num_of_rooms = strtol(argv[1],err, 10);
  if ((strcmp(err[0], "") != 0) || (num_of_rooms < 1)) {
    fprintf(stderr,"Usage: %s #of_rooms\n",argv[0]);
    exit(1);
  }

  
  if (deterministic_flag == 1) {
    sprintf(fname,"det_ring_%s.smv",argv[1]);
  } else {
    sprintf(fname,"nondet_ring_%s.smv",argv[1]);
  }
  if ((fp = fopen(fname,"w")) == NULL) {
    perror(fname);
    exit(1);
  }
  
  if (deterministic_flag == 1) {
    fprintf(fp, "-- Deterministic Ring of Rooms\n");
    fprintf(fp, "-- Called as: %s %s\n",argv[0], argv[1]);
    fprintf(fp, "-- det_ring_%s\nMODULE main\n\n",argv[1]);
  } else {
    fprintf(fp, "-- Nondeterministic Ring of Rooms\n");
    fprintf(fp, "-- Called as: %s %s\n",argv[0], argv[1]);
    fprintf(fp, "-- nondet_ring_%s\nMODULE main\n\n",argv[1]);
  }

  if (deterministic_flag == 1) {
  fprintf(fp, "DEFINE deterministic_flag := TRUE;\n");
  } else {
  fprintf(fp, "DEFINE deterministic_flag := FALSE;\n");
  }

  fprintf(fp, "IVAR\n\taction : { up, dw, open, close, lock, unlock, wait };\n");
  fprintf(fp, "VAR\n\tpos : 0..%d;\n", (num_of_rooms - 1));
  for (i=0;i<num_of_rooms;i++) {
    fprintf(fp, "\twindow_%d : {open, closed};\n", i);
    fprintf(fp, "\tlocked_%d : boolean;\n\n", i);
  }
  

  for (i=0;i<num_of_rooms;i++) {
    fprintf(fp, "
DEFINE can_open_%d := !locked_%d;
DEFINE can_close_%d := !locked_%d;
DEFINE can_lock_%d := !locked_%d;
DEFINE can_unlock_%d := locked_%d;", i,i,i,i,i,i,i,i);
  }

  fprintf(fp, "
ASSIGN
 next(pos) := 
  case
   action = up : (pos + 1) mod %d;
   action = dw : (pos - 1) mod %d;
   1 : pos; 
  esac;
", num_of_rooms, num_of_rooms);


  for (i=0;i<num_of_rooms;i++) {
    fprintf(fp, "
TRANS (pos = %d & !can_open_%d) -> !(action = open)
TRANS (pos = %d & !can_close_%d) -> !(action = close)
TRANS (pos = %d & !can_lock_%d) -> !(action = lock)
TRANS (pos = %d & !can_unlock_%d) -> !(action = unlock)
", i,i,i,i, i,i,i,i);
    fprintf(fp, "
ASSIGN
 next(window_%d) :=
  case
   (pos = %d & action = open) : open;
   (pos = %d & action = close) : closed;
   !deterministic_flag & !(pos = %d) & !locked_%d : {open, closed};
   1 : window_%d;
  esac;
", i,i,i,i,i,i);
    fprintf(fp, "
ASSIGN
 next(locked_%d) :=
  case
   (pos = %d & action = lock) : TRUE;
   (pos = %d & action = unlock) : FALSE;
   1 : locked_%d;
  esac;
", i,i,i,i);
  }

  fprintf(fp, "INIT\n");
  for (i=0;i<num_of_rooms;i++) {
    fprintf(fp, "\t(window_%d = open) &      (locked_%d) &\n",i,i);
  }
  fprintf(fp, "\tTRUE\n");

  fprintf(fp, "NULL_OBS_STRONG_GOAL\n");
  for (i=0;i<num_of_rooms;i++) {
    fprintf(fp, "\t(window_%d = closed) &\n",i);
  }
  fprintf(fp, "\tTRUE\n");

  fclose(fp);
}

